#!/bin/ksh
#
#	$Id: abersnuik.ksh,v 1.35 2014/05/23 10:11:06 craski-shell_86 Exp $
#
# vim: tabstop=4 shiftwidth=4 softtabstop=4 noexpandtab
#
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
# Copyright (c) 2013, 2014 Craig R. Skinner <skinner@britvault.co.uk>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
# VERSION: ${VERSION}
#


# Source the sysexits library:
. ${TRUEPREFIX}/lib/sysexits.sh


function usage
{
	usg="Usage: ${this} {"
	usg="${usg}$(print ${protocols[*]} | tr -s ' ' '|')"
	usg="${usg}} URL"
	usg="${usg}\nRefer to ${TRUEPREFIX}/share/doc/pkg-readmes/${this}*"
	print -u2 ${usg}
	finish ${EX_USAGE} ${usg}
}


function finish
{
	code=$1
	[[ $# -gt 1 ]] &&
	{
		shift
		[[ -w ${log} ]] &&
		{
			print >> ${log}
			date >> ${log}
			print -- $* >> ${log}
		}

		print -- $* | mail -s "${this} error on $(hostname -s)" ${LOGNAME}
	}

	[[ -d ${lock} ]] && rmdir ${lock}
	exit ${code}
}


alias getcap='getcap -f ${conf} -c ${capacity} ${this} 2>/dev/null | cut -d= -f2'
# NOTE: due to a documented bug in getcap(3), semi-colons are replaced with colons:
alias getcap_colon="sed 's/;/:/g'"


this=${0##*/}
user='${ABSNK_USR}'
log="${VARBASE}/log/${this}"
protocols[0]='ftp'
protocols[1]='http'
protocols[2]='rsync'
protocols[3]='scp'
protocols[4]='sftp'
# TODO protocols[5]='ftps'
for prtcl in ${protocols[*]}
do
	[[ ${1} == ${prtcl} ]] &&
	{
		protocol=${prtcl}
		break
	}
done
[[ -n ${protocol} ]] || usage
[[ $# -eq 2 ]] || usage


#Privilege separation:
[[ ${LOGNAME} == ${user} ]] ||
{
	userinfo -e ${user} || finish ${EX_NOUSER} "Unknown system user: '${user}'"

	# Log file creation (as root):
	[[ -f ${log} ]] ||
	{
		[[ -w ${log%/*} ]] || finish ${EX_NOPERM} "Can't make log: ${log}"
		touch ${log}
		chown ${user} ${log}
		chmod 640 ${log}
	}

	# Drop root privileges:
	su -l -s /bin/ksh ${user} $0 $*
	exit $?
}
renice -n 10 $$ >/dev/null


# Log file check as unprivileged user
[[ -w ${log} ]] || finish ${EX_NOPERM} "Can't write to log: ${log}"


conf='${CONFIG_FILE}'
[[ -r ${conf} ]] ||
	finish ${EX_CONFIG} "User '${LOGNAME}' can't read configuration: ${conf}"


# NOTE: due to a documented bug in getcap(3), semi-colons are replaced with colons:
url=$(print ${2} | getcap_colon)
retrieved=$(print ${url} | tr -c '[[:alnum:].\n]' _)


# Set the cache:
capacity='cache'
cache=$(getcap)
[[ -n ${cache} ]] || cache='${LOCALSTATEDIR}'
find ${cache%/*} -type d -perm -o=w -maxdepth 0 2>/dev/null | \
	fgrep -q ${cache%/*} &&
		finish ${EX_CONFIG} "Insecure: World writable cache parent ${cache%/*}"
[[ -d ${cache} ]] ||
{
	mkdir -p ${cache} || finish ${EX_IOERR} "Can't make ${cache}"
}
find ${cache} -type d -perm -o=w -maxdepth 0 2>/dev/null | \
	fgrep -q ${cache} &&
		finish ${EX_CONFIG} "Insecure: World writable cache ${cache}"
[[ -w ${cache} ]] ||
	finish ${EX_IOERR} "Cache '${cache}' not writable by ${LOGNAME}"
cd ${cache} || finish ${EX_IOERR} "Can't cd into ${cache}"


# Exit if another (maybe started earlier)
# instance is trying to get this file too:
lock="${retrieved}.lock"
mkdir ${lock} 2>/dev/null ||
{
	[[ -d ${lock} ]] &&
	{
		# Ignore stale locks more than 1 day old:
		find . -name ${lock} -type d -maxdepth 1 -mtime +1 | \
			fgrep -q ${lock} || exit ${EX_TEMPFAIL}
	}
}
chflags nodump ${lock}
trap 'finish 1 trapped signal: HUP' HUP
trap 'finish 2 trapped signal: INT' INT
trap 'finish 3 trapped signal: QUIT' QUIT
trap 'finish 6 trapped signal: ABRT' ABRT
trap 'finish 15 trapped signal: TERM' TERM
trap 'finish 17 trapped signal: STOP' STOP


# Email an alert that ${DAEMON} isn't running.
${RCDIR}/${DAEMON} check || finish ${EX_UNAVAILABLE} "${DAEMON} isn't running"


# Get the proxy (if any)
# NOTE: due to a documented bug in getcap(3), semi-colons are replaced with colons:
capacity="${protocol}_proxy"
export ${protocol}_proxy=$(getcap | getcap_colon)


# Set the tool to download with:
capacity='tool'
tool=$(getcap)
case ${protocol} in
	http|ftp)
		# Default to ftp if curl not installed:
		# (curl only gets the file if it's newer,
		# with or without a caching proxy)
		tool=$(whence ${tool:-'curl'}) ||
			tool=$(whereis ${tool:-'curl'}) ||
				tool=$(whereis 'ftp')
		;;
	scp|sftp|rsync)
		tool=$(whence ${protocol}) || tool=$(whereis ${protocol})
		;;
esac
[[ -x ${tool} ]] ||
	finish ${EX_UNAVAILABLE} \
		"Tool: ${tool} not found in PATH:${PATH} nor \$sysctl user.cs_path"


# Backup any previously retrieved blacklist - unless rsyncing; update it in place:
[[ -f ${retrieved} && ${protocol} != 'rsync' ]] && mv ${retrieved} ${retrieved}~


# Set the tool's command line args:
case ${tool##*/} in
	ftp)
		tool_args="-v -o ${retrieved} ${protocol}://${url}"
		;;
	lynx)
		tool_args="-stderr -dump ${protocol}://${url} > ${retrieved}"
		# -source flag always returns '0', even if HTTP 404....
		;;
	curl)
		tool_args="--output ${retrieved} --fail --silent --show-error ${protocol}://${url}"
		[[ -f ${retrieved}~ ]] && tool_args="--time-cond ${retrieved}~ ${tool_args}"
		;;
	rsync)
		tool_args="--compress --verbose --temp-dir ${TMPDIR-/tmp} ${url} ${retrieved}"
		;;
	scp)
		tool_args="-B ${url} ${retrieved}"
		;;
	sftp)
		tool_args="${url} ${retrieved}"
		;;
esac


# Do it:
fetch_log=$(mktemp)
print "\nPWD=${PWD}" > ${fetch_log}
env >> ${fetch_log}
for attempt in $(jot 12 0)
do
	sleep $(($RANDOM % 300))
	print >> ${fetch_log}
	date >> ${fetch_log}
	print "${tool} ${tool_args}" >> ${fetch_log}
	eval ${tool} ${tool_args} >> ${fetch_log} 2>&1 &&
	{
		rm ${fetch_log}
		break
	}
done
[[ -f ${fetch_log} ]] &&
{
	cat ${fetch_log} >> ${log}
	rm ${fetch_log}

	# So as not to alert flood,
	# make a lock directory in ${cache} && alert
	# (removed when more than 1 day old)
	find . -name .!alerted -type d -maxdepth 1 -mtime +1 -execdir rmdir -- {} \;
	mkdir .!alerted 2>/dev/null &&
	{
		chflags nodump .!alerted
		print "Error fetching: ${url}\n\nSee ${log}" | \
			mail -s "${this} fetch error on $(hostname -s)" ${LOGNAME}
	}
}


[[ -f ${retrieved} ]] ||
{
	[[ -f ${retrieved}~ ]] || finish ${EX_PROTOCOL} "Not retrieved: ${url}"
	# Load old cached data:
	mv ${retrieved}~ ${retrieved}
}


file_type=$(file -b ${retrieved})
print ${file_type} | fgrep -q gzip &&
{
	extracted=${retrieved%.*}
	[[ ${retrieved} -nt ${extracted} ]] &&
	{
		cp -p ${retrieved} ${retrieved}.$$
		# If extraction fails, load old cached data:
		gunzip -o ${extracted}.$$ ${retrieved} &&
				mv ${extracted}.$$ ${extracted}
		mv ${retrieved}.$$ ${retrieved}
		[[ -f ${extracted} ]] ||
			finish ${EX_DATAERR} "No (cached) extracted ${retrieved}"
	}
	retrieved=${extracted}
	file_type=$(file -b ${retrieved})
}
print ${file_type} | fgrep -q text && reader='cat'


[[ -n ${reader} ]] || finish ${EX_DATAERR} "Don't know how to open ${retrieved}"


egrep -vq '^[[:digit:]]|\#|^$' ${retrieved} &&
	reader="awk '/^[[:digit:]]/ { if (\$1 !~ /127.0.0./ ) print \$1 }'"


eval ${reader} ${retrieved}
finish $?

